package com.hisense.monitormanage.controller;

import com.hisense.monitormanage.dto.WaterCameraDto;
import com.hisense.monitormanage.dto.WaterPointDto;
import com.hisense.monitormanage.dto.WaterPointDtos;
import com.hisense.monitormanage.entity.CameraChannel;
import com.hisense.monitormanage.entity.Result;
import com.hisense.monitormanage.entity.WaterPoint;
import com.hisense.monitormanage.entity.WaterPointPolice;
import com.hisense.monitormanage.mapper.WaterPointMapper;
import com.hisense.monitormanage.utils.LongLatUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("api/project/water")
@Api(tags = "积水点")
public class WaterPointController {
    @Autowired
    private WaterPointMapper waterPointMapper;
    @Autowired
    JdbcTemplate jdbcTemplate;

    /**
     * 查询积水点列表
     *
     * @return
     */
    @GetMapping("selectWaterPoint")
    @ApiOperation("查询积水点列表")
    public Result selectWaterPoint() {
        List<WaterPointDto> waterPoints = waterPointMapper.selectWaterPoint();
        Result success = Result.success(waterPoints);
        return success;
    }

    /**
     * 查询积水点等级
     *
     * @param reportOrigin
     * @return
     */
    @GetMapping("selectReportOrigin")
    @ApiOperation("查询积水点等级")
    @ApiImplicitParam(name = "reportOrigin", value = "等级标识", required = true, dataType = "String")
    public Result selectReportOrigin(String reportOrigin) {
        List<WaterPointDto> waterPoints = waterPointMapper.selectReportOrigin(reportOrigin);
        Result success = Result.success(waterPoints);
        return success;
    }

    /**
     * 根据名称查询积水点详细信息
     *
     * @param waterPointName
     * @return
     */
    @GetMapping("selectCameraByName")
    @ApiOperation("根据名称查询积水点详细信息")
    @ApiImplicitParam(name = "waterPointName", value = "积水点名称", required = true, dataType = "String")
    public Result selectCameraByName(String waterPointName) {
        List<WaterCameraDto> waterCameras = waterPointMapper.selectCameraByName(waterPointName);
        Result success = Result.success(waterCameras);
        return success;
    }

    /**
     * 统计分析积水点市区分布
     *
     * @return
     */
    @GetMapping("selectCountWater")
    @ApiOperation("统计分析积水点市区分布")
    public Result selectCountWater() {
        List<WaterPointDtos> waterCameras = waterPointMapper.selectCountWater();
        Result success = Result.success(waterCameras);
        return success;
    }

    /**
     * 统计分析积水点来源
     *
     * @return
     */
    @GetMapping("selectCountWaters")
    @ApiOperation("统计分析积水点来源")
    public Result selectCountWaters() {
        List<WaterPointDtos> waterCameras = waterPointMapper.selectCountWaters();
        Result success = Result.success(waterCameras);
        return success;
    }

    /**
     * 统计分析积水点等级
     *
     * @return
     */
    @GetMapping("selectCountLevel")
    @ApiOperation("统计分析积水点等级")
    public Result selectCountLevel() {
        List<WaterPointDtos> waterCameras = waterPointMapper.selectCountLevel();
        Result success = Result.success(waterCameras);
        return success;
    }

    /**
     * 根据经纬度查询积水点周边监控
     *
     * @param longitude
     * @param latitude
     * @param radius
     * @return
     */
    @GetMapping("selectByLongAndLiti")
    @ApiOperation("根据经纬度查询积水点周边监控")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "longitude", value = "经度", required = true, dataType = "Double"),
            @ApiImplicitParam(name = "latitude", value = "纬度", required = true, dataType = "Double"),
            @ApiImplicitParam(name = "radius", value = "半径，米", required = true, dataType = "Integer"),}
    )
    public Result selectByLongAndLiti(Double longitude, Double latitude, Integer radius) {
        double[] around = LongLatUtil.getAround(longitude, latitude, radius);
        List<WaterPoint> c = waterPointMapper.selectByLongAndLiti(around[0], around[2], around[1], around[3]);
        Result success = Result.success(c);
        return success;
    }

    /**
     * 查询交警提供积水点列表
     *
     * @return
     */
    @GetMapping("selectWaterPoliceList")
    @ApiOperation("查询交警提供积水点列表")
    public Result selectWaterPoliceList() {
        List<WaterPointPolice> pointPolices = waterPointMapper.selectWaterPoliceList();
        Result success = Result.success(pointPolices);
        return success;
    }


    /**
     * 根据积水点查询该点存在积水情况的日期
     *
     * @param water_point_id
     * @return
     */
    @GetMapping("selectWaterPondingDate")
    @ApiOperation("查询积水点存在积水的日期")
    public Result selectWaterPondingDate(@RequestParam String water_point_id) {
        if (StringUtils.isEmpty(water_point_id)) {
            return Result.error("未提供积水点id");
        }
        String sql = String.format("SELECT DISTINCT " +
                "YEAR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) `year`," +
                "MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) `month`," +
                "DAY ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) `day` " +
                " FROM " +
                "water_point_sensor " +
                " WHERE " +
                "water_point_sensor.water_point_id = '%s' " +
                "AND water_point_sensor.updated_date != '0.00'" +
                " ORDER BY " +
                "`year` DESC," +
                "`month` DESC," +
                "`day` DESC;", water_point_id);
        return Result.success(jdbcTemplate.queryForList(sql));
    }


    /**
     * 查询当天的积水点深度情况
     *
     * @param water_point_id
     * @param year
     * @param month
     * @param day
     * @return
     */
    @GetMapping("selectWaterPondingInDay")
    @ApiOperation("查询积水点当天的积水深度")
    public Result selectWaterPondingInDay(@RequestParam String water_point_id, @RequestParam String year, @RequestParam String month, @RequestParam String day) {
        if (StringUtils.isEmpty(water_point_id)) {
            return Result.error("未提供积水点id");
        }
        if (StringUtils.isEmpty(year) || StringUtils.isEmpty(month) || StringUtils.isEmpty(day)) {
            return Result.error("未提供日期");
        }
        String sql = String.format("SELECT " +
                "updated_date AS depth," +
                "YEAR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `year`," +
                "MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `month`," +
                "DAY ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `day`," +
                "HOUR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' )) AS `hour`," +
                "MINUTE ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' )) AS `minute`" +
                " FROM " +
                " water_point_sensor " +
                " WHERE " +
                "water_point_sensor.water_point_id = '%s' " +
                "AND YEAR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) = %s " +
                "AND MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) = %s " +
                "AND DAY ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) = %s " +
                " ORDER BY " +
                "`year` DESC," +
                "`month` DESC," +
                "`day` DESC," +
                "`hour`," +
                "`minute`;", water_point_id, year, month, day);
        return Result.success(jdbcTemplate.queryForList(sql));
    }

    /**
     * 查询积水点当月内每天最深深度
     *
     * @param water_point_id
     * @param year
     * @param month
     * @return
     */
    @GetMapping("selectWaterPondingInMonth")
    @ApiOperation("查询积水点当月内最深深度")
    public Result selectWaterPondingInMonth(@RequestParam String water_point_id, @RequestParam String year, @RequestParam String month) {
        if (StringUtils.isEmpty(water_point_id)) {
            return Result.error("未提供积水点id");
        }
        if (StringUtils.isEmpty(year) || StringUtils.isEmpty(month)) {
            return Result.error("未提供日期");
        }
        String sql = String.format("SELECT " +
                "MAX( updated_date ) AS depth," +
                "YEAR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `year`," +
                "MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `month`," +
                "DAY ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `day` " +
                " FROM " +
                "water_point_sensor " +
                " WHERE " +
                "water_point_sensor.water_point_id = '%s' " +
                "AND YEAR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) = %s " +
                "AND MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) = %s " +
                " GROUP BY " +
                "`year`," +
                "`month`," +
                "`day` " +
                " ORDER BY " +
                "`year` DESC," +
                "`month` DESC," +
                "`day`;", water_point_id, year, month);
        return Result.success(jdbcTemplate.queryForList(sql));
    }


    /**
     * 查询积水点当年每月积水时间
     *
     * @param water_point_id
     * @param year
     * @return
     */
    @GetMapping("selectWaterPondingHoursInYear")
    @ApiOperation("查询积水点当年每月积水时间")
    public Result selectWaterPondingHoursInYear(@RequestParam String water_point_id, @RequestParam String year) {
        if (StringUtils.isEmpty(water_point_id)) {
            return Result.error("未提供积水点id");
        }
        if (StringUtils.isEmpty(year)) {
            return Result.error("未提供日期");
        }

        String sql = String.format("SELECT " +
                "`year`," +
                "`month`," +
                "COUNT( `hour` ) AS `hours` " +
                " FROM " +
                "(SELECT DISTINCT YEAR( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `year`," +
                "MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `month`," +
                "DAY ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `day`," +
                "HOUR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' )) AS `hour`" +
                " FROM " +
                "water_point_sensor " +
                " WHERE " +
                "water_point_sensor.water_point_id = '%s' " +
                " AND water_point_sensor.updated_date != '0.00' " +
                " AND YEAR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) = %s " +
                " GROUP BY " +
                "`year`," +
                "`month`," +
                "`day`," +
                "`hour` " +
                " ORDER BY " +
                "`year` DESC," +
                "`month` DESC," +
                "`day` DESC " +
                ") AS temp " +
                " GROUP BY " +
                " `year`," +
                " `month` " +
                " ORDER BY " +
                " `month`;", water_point_id, year);
        return Result.success(jdbcTemplate.queryForList(sql));
    }

    /**
     * 查询积水点当年某月每天积水时间
     *
     * @param water_point_id
     * @param year
     * @return
     */
    @GetMapping("selectWaterPondingHoursInMonth")
    @ApiOperation("查询积水点当年某月每天积水时间")
    public Result selectWaterPondingHoursInMonth(@RequestParam String water_point_id, @RequestParam String year, @RequestParam String month) {
        if (StringUtils.isEmpty(water_point_id)) {
            return Result.error("未提供积水点id");
        }
        if (StringUtils.isEmpty(year) || StringUtils.isEmpty(month)) {
            return Result.error("未提供日期");
        }

        String sql = String.format("SELECT  " +
                "  `year`,  " +
                "  `month`,  " +
                "  `day`,  " +
                "  COUNT( `hour` ) AS `hours`   " +
                "FROM  " +
                "  (  " +
                "  SELECT DISTINCT YEAR  " +
                "    ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `year`,  " +
                "    MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `month`,  " +
                "    DAY ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) AS `day`,  " +
                "    HOUR (  " +
                "    STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' )) AS `hour`   " +
                "  FROM  " +
                "    water_point_sensor   " +
                "  WHERE  " +
                "    water_point_sensor.water_point_id = '%s'   " +
                "    AND water_point_sensor.updated_date != '0.00'   " +
                "    AND YEAR ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) =  %s  " +
                "    AND MONTH ( STR_TO_DATE( water_point_sensor.water_point_area, '%%Y-%%m-%%d %%H:%%i:%%s' ) ) = %s   " +
                "  GROUP BY  " +
                "    `year`,  " +
                "    `month`,  " +
                "    `day`,  " +
                "    `hour`   " +
                "  ORDER BY  " +
                "    `year` DESC,  " +
                "    `month` DESC,  " +
                "    `day` DESC   " +
                "  ) AS temp   " +
                "GROUP BY  " +
                "  `year`,  " +
                "  `month`,  " +
                "  `day`   " +
                "ORDER BY  " +
                "  `month`,  " +
                "  `day`;", water_point_id, year, month);
        return Result.success(jdbcTemplate.queryForList(sql));
    }

}
